#!/usr/bin/env python
# IMPORTANT
#
# We cannot do partial requests to Rules API, so we will need to:
# - make update for groups which has been modified
# - updated groups must be posted in full
#
# Script consists of 3 stages (for each rule namespace):
# 1. compare new and old rules
#     - old delivered with previous Stacklight release
#     - new delivered with new Stacklight release
#     - diff means "release update"
# 2. compare current (retrieved from API) with old rules
#     - diff means "manually changed"
# 3. compare release update with manually changed
#     - diff means "update to apply to current rules"

from argparse import ArgumentParser
from collections import OrderedDict
import os
import sys

import oyaml as yaml


# not included namespaces are ignored
NAMESPACES = {
    'stacklight-alerts': 'alert',
    'stacklight-recording-rules': 'record'
}


def yaml_as_dict(filename):
    result_dict = {}
    with open(filename, 'r') as fp:
        for doc in yaml.safe_load_all(fp):
            for key, value in doc.items():
                result_dict[key] = value
    return result_dict


def merge_cm_into_dict(cm_path, namespace):
    result_dict = {namespace: []}
    for filename in os.listdir(cm_path):
        file_path = os.path.join(os.path.abspath(cm_path), filename)
        if os.path.isdir(file_path):
            sys.stdout.write(
                'Found directory {}. Skipping\n'.format(file_path))
            continue
        result_dict[namespace].append(yaml_as_dict(file_path))
    return result_dict


def save_as_yaml(input_data, filename):
    with open(filename, 'w+') as fp:
        fp.write(yaml.dump(input_data))


def list_to_dict(input_list, keyname):
    dict_of_dicts = {}
    for input_elem in input_list:
        sorted_elem = sorted(list(input_elem.items()), key=lambda x: x[0])
        new_elem = OrderedDict(sorted_elem)
        dict_of_dicts[new_elem[keyname]] = new_elem
    return dict_of_dicts


def dict_to_list(input_dict):
    # actually it's list of dicts with list under one key
    # like [{'name': x, 'rules': []}]
    list_of_lists = []
    for _, group_body in input_dict.items():
        group_body['rules'] = list(group_body['rules'].values())
        list_of_lists.append(group_body)
    return list_of_lists


def simplify_struct_for_update(rules_content, namespace, keyname='alert'):

    # handle initial deployment where no rules are deployed
    if rules_content == {}:
        return {}

    ns_dict = list_to_dict(rules_content[namespace], 'name')

    for grp_name, group in ns_dict.items():
        ns_dict[grp_name]['rules'] = list_to_dict(group['rules'], keyname)

    return ns_dict


def nested_dict_to_sorted_tuple(dict_obj):
    new_dict = {}
    for k, v in dict_obj.items():
        if isinstance(v, dict):
            new_dict[k] = tuple(sorted(list(v.items()), key=lambda x: x[0]))
        else:
            new_dict[k] = v
    return tuple(sorted(list(new_dict.items()), key=lambda x: x[0]))


def rules_difference(new_group, old_group, keyname):
    # requires list_to_dict for args
    added_rules = set(new_group.keys()).difference(
        set(old_group.keys()))
    removed_rules = set(old_group.keys()).difference(
        set(new_group.keys()))
    modified_rules = {
        nested_dict_to_sorted_tuple(v) for v in new_group.values()
        }.difference({
        nested_dict_to_sorted_tuple(v) for v in old_group.values()
        })
    modified_rules = {dict(v)[keyname] for v in modified_rules}

    return {
        'added': {k: v for k, v in new_group.items()
                  if k in added_rules.union(modified_rules)},
        'removed': {k: v for k, v in old_group.items()
                    if k in removed_rules},
    }


def groups_difference(new_namespace, old_namespace):
    # requires list_to_dict for args
    added_groups = set(new_namespace.keys()).difference(
        set(old_namespace.keys()))
    removed_groups = set(old_namespace.keys()).difference(
        set(new_namespace.keys()))

    return {
        'added': {k: v for k, v in new_namespace.items()
                  if k in added_groups},
        'removed': {k: v for k, v in old_namespace.items()
                    if k in removed_groups},
    }


def namespaces_diff(new_ns, old_ns, namespace, keyname='alert'):

    groups_diff = groups_difference(new_ns, old_ns)
    for grp_name, group in new_ns.items():
        if grp_name not in groups_diff['added']:
            new_grp = group['rules']
            old_grp = old_ns[grp_name]['rules']
            rules_diff = rules_difference(new_grp, old_grp, keyname)

            for operation in ['added', 'removed']:
                if rules_diff[operation]:
                    groups_diff[operation][grp_name] = {
                        'name': grp_name,
                        'rules': rules_diff[operation]
                    }

    return groups_diff


def merge_updates(release_update, manual_update, release_ns):
    # ASSUMPTIONS
    # 1. manual changes take precedence over release updates
    #    - for addition:
    #      - all additions done manually override release update changes
    #    - for removal:
    #      - manual removal overrides release update
    #      - don't remove group if at least one rule was modified
    #      - if the group contains "Reset"-named rule - reset the group
    #        to release default state
    # 2. adding {reset: true} rule label "resets" rule to release update

    # resolve conflicts for additions
    for group, group_body in manual_update['added'].items():

        release_rules = release_ns[group]['rules']
        group_reset = {}

        # override release update with manual changes
        for rule, rule_body in group_body.get('rules', {}).items():

            if release_update['added'].get(group) is None:
                release_update['added'][group] = {
                    'name': group,
                    'rules': {}
                }

            # if 'Reset' rule is present - reset whole group rules
            if rule == 'Reset':
                release_update['added'][group]['rules'] = release_rules
                group_reset[group] = True

                # remove 'Reset' rule
                manual_update['removed'][group] = {'rules': {'Reset': {}}}
                break

            if rule_body.get('labels', {}).get('reset', '') == 'true':
                # in case custom alert was created in product group
                # with reset label - avoid breaking whole script
                rule_body = release_rules.get(rule, rule_body)
            release_update['added'][group]['rules'][rule] = rule_body

        if group_reset.get(group, False):
            break

        # rules removed in release, but manually changed - leave group
        removed_from_release = release_update['removed'].get(group)
        if removed_from_release is not None:
            # release_update['added'][group] will exist after previous step
            # if only at least one manual non-resetting change was introduced
            if release_update['added'].get(group) is None:
                for rule, rule_body in removed_from_release.get(
                        'rules', {}).items():
                    rules = release_update['added'][group]['rules']
                    rules[rule] = rules.get(rule) or rule_body
                del release_update['removed'][group]

    # resolve conflicts for removal
    for group, group_body in manual_update['removed'].items():

        # manually removed group
        if release_update['added'].get(group) is None:
            release_update['removed'][group] = group_body
            continue

        # if rule was removed manually - remove it from update
        for rule, rule_body in group_body['rules'].items():
            if release_update['added'].get(group, {}).get(
                    'rules', {}).get(rule) is not None:
                del release_update['added'][group]['rules'][rule]

            if release_update['removed'].get(group) is None:
                release_update['removed'][group] = {
                    'name': group,
                    'rules': {}
                }
            release_update['removed'][group]['rules'][rule] = rule_body

    return release_update


def rm_unchanged_groups(rules_dict, update_dict):
    changed_groups = list(update_dict['added'].keys())
    changed_groups.extend(list(update_dict['removed'].keys()))
    sys.stdout.write('Changed groups: {}\n'.format(set(changed_groups)))

    return {group: v for group, v in rules_dict.items()
            if group in changed_groups}


def apply_update(rules_dict, update_dict):

    for group, group_body in update_dict['added'].items():
        if rules_dict.get(group) is None:
            rules_dict[group] = {
                'name': group,
                'rules': {}
            }
        for rule, rule_body in group_body['rules'].items():
            rules_dict[group]['rules'][rule] = rule_body

    for group, group_body in update_dict['removed'].items():
        for rule, rule_body in group_body['rules'].items():
            if rules_dict.get(group, {}).get('rules', {}).get(
                    rule) is not None:
                del rules_dict[group]['rules'][rule]

    return rules_dict


def create_updated_rules(namespace, new_cm, old_cm, current, output):
    keyname = NAMESPACES.get(namespace)

    if keyname is None:
        sys.stdout.write(('Not valid namespace name '
                          '(allowed {}). Exiting\n').format(
                         list(NAMESPACES.keys())))
        return

    sys.stdout.write('Processing {} namespace.\n'.format(namespace))
    new_cm = merge_cm_into_dict(new_cm, namespace)
    old_cm = merge_cm_into_dict(old_cm, namespace)
    current = yaml_as_dict(current)

    new_ns = simplify_struct_for_update(new_cm, namespace, keyname)
    old_ns = simplify_struct_for_update(old_cm, namespace, keyname)
    current_ns = simplify_struct_for_update(current, namespace, keyname)

    updated_rules = namespaces_diff(new_ns, old_ns, namespace, keyname)

    man_changed_rules = namespaces_diff(current_ns, old_ns, namespace, keyname)

    # no need to upload manually added groups - remove them from update
    release_groups = set(old_ns.keys()).union(new_ns.keys())
    man_added_groups = set(group
                           for group in man_changed_rules['added'].keys()
                           if group not in release_groups)

    for group in man_added_groups:
        man_changed_rules['added'].pop(group)

    update = merge_updates(updated_rules, man_changed_rules,
                           new_ns)
    current_ns = rm_unchanged_groups(current_ns, update)
    updated_current = apply_update(current_ns, update)

    result_filename = '{}/{{name}}.yaml'.format(output)
    for group in dict_to_list(updated_current):
        if group['rules']:
            save_as_yaml(group, result_filename.format(**group))


def parse_args():
    parser = ArgumentParser()
    parser.add_argument('--old_cm', default='/tmp/old-rules',
                        help='path old rules configMap mount')
    parser.add_argument('--new_cm', default='/tmp/new-rules',
                        help='path new rules configMap mount')
    parser.add_argument('--current', default='/tmp/api-rules.yaml',
                        help='path Ruler API dump')
    parser.add_argument('--output', default='/tmp/output',
                        help='directory to place rule files to apply')
    parser.add_argument('--namespace', default='stacklight-alerts',
                        help='rules namespace to process')
    return parser.parse_args()


def main():
    args = parse_args()

    create_updated_rules(args.namespace, args.new_cm, args.old_cm,
                         args.current, args.output)


if __name__ == '__main__':
    main()
