#!/usr/bin/env python

import json
import os
import sys
import time

import requests


K8S_API_URL = os.environ.get('K8S_API_URL')
K8S_TOKEN_PATH = os.environ.get('K8S_TOKEN_PATH')
RULER = os.environ.get('RULER_API_URL')


with open(K8S_TOKEN_PATH) as token:
    K8S_TOKEN = token.read()


K8S_API_HEADERS = {
    'Authorization': 'Bearer {}'.format(K8S_TOKEN),
    'Content-Type': 'application/json'
}


def k8s_requests(http_request, kind, namespace, **kwargs):
    if namespace != '':
        namespace = 'namespaces/{}/'.format(namespace)

    api = 'api'
    if kind == 'statefulset':
        api = 'apis/apps'

    headers = K8S_API_HEADERS

    if http_request == 'get':
        selector = kwargs['selector']
        if selector[0] != '?':
            # assume selection by name
            selector = '/{}'.format(selector)

        k8s_url_fmt = '{}/{}/v1/{}{}s{}'
        k8s_url = k8s_url_fmt.format(K8S_API_URL, api,
                                     namespace, kind, selector)

        return requests.get(k8s_url, headers=headers)
    elif http_request == 'post':
        headers.update({
            'Accept': 'application/json'
        })
        data = kwargs['data']

        k8s_url_fmt = '{}/{}/v1/{}{}s'
        k8s_url = k8s_url_fmt.format(K8S_API_URL, api, namespace, kind)

        return requests.post(k8s_url, headers=headers, data=data)
    elif http_request == 'patch':
        headers.update({
            'Accept': 'application/json',
            'Content-Type': 'application/json-patch+json'
        })
        name = kwargs['name']
        data = kwargs['data']

        k8s_url_fmt = '{}/{}/v1/{}{}s/{}'
        k8s_url = k8s_url_fmt.format(K8S_API_URL, api, namespace, kind, name)

        return requests.patch(k8s_url, headers=headers, data=data)


def get_resource(kind, namespace, selector=None, skip_404=False):

    msg_fmt = 'Getting {} {} in {} namespace {}.\n'

    response = k8s_requests('get', kind, namespace,
                            selector=selector, skip_404=skip_404)

    if response.status_code != 200:
        if response.status_code == 404:
            if skip_404:
                sys.stdout.write(msg_fmt.format(selector, kind, namespace,
                                 'succeeded with 404'))
                return {}
        sys.stdout.write(msg_fmt.format(selector, kind, namespace,
                         'failed'))
        sys.exit(1)
    sys.stdout.write(msg_fmt.format(selector, kind, namespace, 'succeeded'))
    return response.json()


def create_resource(kind, namespace, data=None):

    response = k8s_requests('post', kind, namespace, data=json.dumps(data))

    name = data['metadata']['name']
    msg_fmt = 'Creating {} {} in {} namespace {}.\n'

    if response.status_code not in [200, 201]:
        sys.stdout.write(msg_fmt.format(name, kind, namespace, 'failed'))
        sys.exit(1)

    sys.stdout.write(msg_fmt.format(name, kind, namespace, 'succeeded'))


def update_resource(kind, namespace, name=None, key=None, data=None):

    data = [{
        "op": "replace",
        "path": key,
        "value": data
    }]

    response = k8s_requests('patch', kind, namespace, name=name,
                            data=json.dumps(data))

    msg_fmt = 'Updating {} {} in {} namespace {}.\n'
    if response.status_code not in [200, 201]:
        sys.stdout.write(msg_fmt.format(name, kind, namespace, 'failed'))
        sys.exit(1)

    sys.stdout.write(msg_fmt.format(name, kind, namespace, 'succeeded'))


def sync_rules_configmaps(new_cm_name, old_cm_name):
    new_cm_json = get_resource('configmap', 'stacklight',
                               selector=new_cm_name)
    old_cm_json = get_resource('configmap', 'stacklight',
                               selector=old_cm_name, skip_404=True)

    if not old_cm_json:
        for field in ['creationTimestamp', 'managedFields',
                      'resourceVersion', 'revisionVersion', 'uid']:
            if field in new_cm_json['metadata'].keys():
                del new_cm_json['metadata'][field]
        new_cm_json['metadata']['name'] = old_cm_name

        create_resource('configmap', 'stacklight', data=new_cm_json)
    else:
        update_resource('configmap', 'stacklight', name=old_cm_name,
                        key='/data', data=new_cm_json['data'])


def wait_for_endpoint(http_url):
    response = requests.get(http_url)
    while response.status_code != 200:
        msg = 'Endpoint {} not available. Waiting 10 seconds and retrying.\n'
        sys.stdout.write(msg.format(http_url))
        time.sleep(10)
        try:
            response = requests.get(http_url)
        except:
            pass


def get_rules_from_api(namespace, output_file):
    sys.stdout.write('Waiting for Ruler API\n')
    wait_for_endpoint(RULER)

    sys.stdout.write('Downloading all rules configuration\n')
    response = requests.get('{}/prometheus/config/v1/rules/{}'.format(RULER, namespace))

    msg_fmt = 'Downloading all {} namespace rules {}.\n'

    if response.status_code != 200:
        sys.stdout.write(msg_fmt.format(namespace, 'failed'))
        sys.exit(1)

    with open(output_file, 'w+') as f:
        f.write(response.text)
    sys.stdout.write(msg_fmt.format(namespace, 'succeeded'))


def upload_rules_changes(rules_directory, namespace):
    sys.stdout.write('Waiting for Ruler API\n')
    wait_for_endpoint(RULER)

    msg_fmt = 'Loading {} rules to {} namespace {}.\n'

    for rules_file in os.listdir(rules_directory):
        with open('{}/{}'.format(rules_directory, rules_file), 'rb') as f:
            rules_data = f.read()

        response = requests.post(
            '{}/prometheus/config/v1/rules/{}'.format(RULER, namespace),
            data=rules_data)

        rules_group = rules_file.split('.yaml')[0]

        if response.status_code not in [200, 201, 202]:
            sys.stdout.write(msg_fmt.format(rules_group, namespace, 'failed'))
            sys.exit(1)

        sys.stdout.write(msg_fmt.format(rules_group, namespace, 'succeeded'))
