{{- define "prometheus.alertmanager" -}}
{{- $logLevels := dict "trace" "debug" "debug" "debug" "info" "info" "warn" "warn" "error" "error" "crit" "error" -}}
{{- $logLevelKey := index (index .Values.stacklightLogLevels "component" | default dict) "prometheusAlertManager" | default (index .Values.stacklightLogLevels "default" | default "") -}}
{{- $logLevelValue := index $logLevels $logLevelKey | default "" -}}
alertmanager:
  enabled: {{ not .Values.cortexDev.enabled }}
  affinity:
    podAntiAffinity:
  {{- if .Values.highAvailabilityEnabled }}
      requiredDuringSchedulingIgnoredDuringExecution:
      - labelSelector:
          matchExpressions:
          - key: app
            operator: In
            values:
            - "prometheus"
          - key: component
            operator: In
            values:
            - "alertmanager"
        topologyKey: "kubernetes.io/hostname"
  {{- else }}
      preferredDuringSchedulingIgnoredDuringExecution:
      - weight: 100
        podAffinityTerm:
          labelSelector:
            matchExpressions:
            - key: app
              operator: In
              values:
              - "prometheus"
            - key: component
              operator: In
              values:
              - "alertmanager"
          topologyKey: "kubernetes.io/hostname"
  {{- end }}
  replicaCount: {{ get (include "stacklight.replicas" . | fromYaml) "alertmanager" }}
  {{- if $logLevelValue }}
  extraArgs:
    "log.level": {{ $logLevelValue | quote }}
  {{- end }}
  statefulSet:
    headless:
      enableMeshPeer: true
  image:
    {{- include "stacklight.getImage" (dict "root" . "image" "alertmanager") | nindent 4 }}
  {{- if or .Values.storage.defaultStorageClass .Values.storage.componentStorageClasses.prometheusAlertManager }}
    {{- $storageClass := .Values.storage.defaultStorageClass }}
    {{- if .Values.storage.componentStorageClasses.prometheusAlertManager }}
      {{- $storageClass = .Values.storage.componentStorageClasses.prometheusAlertManager }}
    {{- end }}
  persistentVolume:
    storageClass: "{{ $storageClass }}"
  {{- end }}
  {{- if .Values.exposeLoadBalancers }}
  service:
    type: LoadBalancer
  {{- end }}
  {{- include "stacklight.resources" (dict "inTplValues" .Values "service" "alertmanager") | nindent 2 }}
  {{- include "stacklight.nodeSelector" (dict "inTplValues" .Values "service" "prometheusAlertManager") | nindent 2 }}
  {{- include "stacklight.tolerations" (dict "inTplValues" .Values "service" "prometheusAlertManager") | nindent 2 }}
{{- end }}

{{- define "prometheus.alertmanagerFiles" -}}
{{- $files := .Files }}
{{- $amConfig := .Values.alertmanagerSimpleConfig }}
{{- if $amConfig.enabled -}}
{{- $routesEnabled := or (and .Values.alerta.enabled $amConfig.alerta.enabled) $amConfig.salesForce.enabled $amConfig.serviceNow.enabled $amConfig.slack.enabled $amConfig.email.enabled $amConfig.genericRoutes }}
alertmanagerFiles:
  alertmanager.yml:
    global:
      resolve_timeout: 5m
    route:
      receiver: default
      group_by:
        - alertname
        - instance
  {{- if $routesEnabled }}
      routes:
    {{- if and .Values.alerta.enabled $amConfig.alerta.enabled }}
        - receiver: HTTP-alerta
          continue: true
    {{- end }}
    {{- if $amConfig.msteams.enabled }}
        - receiver: HTTP-msteams
          continue: true
          {{- include "stacklight.alertmanager.route" $amConfig.msteams.route | nindent 10 }}
    {{- end }}
    {{- if $amConfig.salesForce.enabled }}
        - receiver: HTTP-salesforce
          continue: true
          {{- include "stacklight.alertmanager.route" $amConfig.salesForce.route | nindent 10 }}
    {{- end }}
    {{- if $amConfig.serviceNow.enabled }}
        - receiver: HTTP-servicenow
          continue: true
          {{- include "stacklight.alertmanager.route" $amConfig.serviceNow.route | nindent 10 }}
    {{- end }}
    {{- if $amConfig.slack.enabled }}
        - receiver: HTTP-slack
          continue: true
          {{- include "stacklight.alertmanager.route" $amConfig.slack.route | nindent 10 }}
    {{- end }}
    {{- if $amConfig.email.enabled }}
        - receiver: Email
          continue: true
          {{- include "stacklight.alertmanager.route" $amConfig.email.route | nindent 10 }}
    {{- end }}
    {{- range $route := $amConfig.genericRoutes }}
      {{- if or (not (hasKey $route "enabled")) (index $route "enabled") }}
        - {{ unset $route "enabled" | toYaml | nindent 10 }}
      {{- end }}
    {{- end }}
  {{- end }}
      group_wait: 1m
      group_interval: 5m
      repeat_interval: {{ $amConfig.repeatInterval }}
    receivers:
      - name: default
    {{- if and .Values.alerta.enabled $amConfig.alerta.enabled }}
      - name: HTTP-alerta
        webhook_configs:
        - send_resolved: true
          url: http://alerta/api/webhooks/prometheus
          http_config: {}
    {{- end }}
    {{- if $amConfig.msteams.enabled }}
      - name: HTTP-msteams
        webhook_configs:
        - send_resolved: {{ $amConfig.msteams.send_resolved }}
          url: http://prometheus-msteams:2000/alertmanager
          http_config: {}
    {{- end }}
    {{- if $amConfig.salesForce.enabled }}
      - name: HTTP-salesforce
        webhook_configs:
        - send_resolved: true
          url: http://sf-notifier:5000/hook
          http_config: {}
    {{- end }}
    {{- if $amConfig.serviceNow.enabled }}
      - name: HTTP-servicenow
        webhook_configs:
        - send_resolved: true
          url: http://alertmanager-webhook-servicenow:5000/hook
          http_config: {}
    {{- end }}
    {{- if $amConfig.slack.enabled }}
      - name: HTTP-slack
        slack_configs:
        - send_resolved: {{ $amConfig.slack.send_resolved }}
          api_url: {{ $amConfig.slack.api_url }}
          channel: {{ $amConfig.slack.channel }}
          text: '{{ "{{ template \"slack.notification.text\" . }}" }}'
          title: '{{ "{{ template \"slack.notification.title\" . }}" }}'
    {{- end }}
    {{- if $amConfig.email.enabled }}
      - name: Email
        email_configs:
          - send_resolved: {{ $amConfig.email.send_resolved }}
            to: {{ $amConfig.email.to }}
            from: {{ $amConfig.email.from }}
            smarthost: {{ $amConfig.email.smarthost }}
            text: '{{ "{{ template \"email.notification.text\" . }}" }}'
            html: '{{ "{{ template \"email.notification.html\" . }}" }}'
            headers:
              subject: '{{ "{{ template \"email.notification.subject\" . }}" }}'
      {{- if $amConfig.email.auth_username }}
            auth_username: {{ $amConfig.email.auth_username }}
      {{- end }}
      {{- if $amConfig.email.auth_password }}
            auth_password: {{ $amConfig.email.auth_password }}
      {{- end }}
      {{- if $amConfig.email.auth_secret }}
            auth_secret: {{ $amConfig.email.auth_secret }}
      {{- end }}
      {{- if $amConfig.email.auth_identity }}
            auth_identity: {{ $amConfig.email.auth_identity }}
      {{- end }}
            require_tls: {{ $amConfig.email.require_tls }}
    {{- end }}
    {{- range $receiver := $amConfig.genericReceivers }}
      {{- if or (not (hasKey $receiver "enabled")) (index $receiver "enabled") }}
      - {{ unset $receiver "enabled" | toYaml | nindent 8 }}
      {{- end }}
    {{- end }}

    {{- if or $amConfig.slack.enabled $amConfig.email.enabled }}
    templates:
      {{- if $amConfig.slack.enabled }}
      - /etc/config/slack-title
      - /etc/config/slack-text
      {{- end }}
      {{- if $amConfig.email.enabled }}
      - /etc/config/email-subject
      - /etc/config/email-html
      - /etc/config/email-text
      {{- end }}
    {{- else }}
    templates: []
    {{- end }}
    {{- if .Values.alertmanagerSimpleConfig.inhibitRules.enabled }}
    inhibit_rules:
    {{- include "stacklight.collectAllInhibitRules" . | nindent 4 }}
    {{- end }}
  {{- if $amConfig.slack.enabled }}
  slack-title:
    {{- if $amConfig.slack.customTemplates.title }}
    {{ $amConfig.slack.customTemplates.title | toYaml | indent 2 }}
    {{- else }}
    {{ $files.Get "files/notifications/slack/title" | toYaml | indent 2 }}
    {{- end }}
  slack-text:
    {{- if $amConfig.slack.customTemplates.text }}
    {{ $amConfig.slack.customTemplates.text | toYaml | indent 2 }}
    {{- else }}
    {{ $files.Get "files/notifications/slack/text" | toYaml | indent 2 }}
    {{- end }}
  {{- end }}
  {{- if $amConfig.email.enabled }}
  email-subject:
    {{- if $amConfig.email.customTemplates.subject }}
    {{ $amConfig.email.customTemplates.subject | toYaml | indent 2 }}
    {{- else }}
    {{ $files.Get "files/notifications/email/subject" | toYaml | indent 2 }}
    {{- end }}
  email-html:
    {{- if $amConfig.email.customTemplates.html }}
    {{ $amConfig.email.customTemplates.html | toYaml | indent 2 }}
    {{- else }}
    {{ $files.Get "files/notifications/email/html" | toYaml | indent 2 }}
    {{- end }}
  email-text:
    {{- if $amConfig.email.customTemplates.text }}
    {{ $amConfig.email.customTemplates.text | toYaml | indent 2 }}
    {{- else }}
    {{ $files.Get "files/notifications/email/text" | toYaml | indent 2 }}
    {{- end }}
  {{- end }}
{{- end }}
{{- end }}

{{- define "prometheus.alertmanager.httpProxy" -}}
httpProxyDiscovery:
  enabled: {{ .Values.global.proxy.enabled }}
  secretName: {{ .Values.global.proxy.secretName }}
  checksum: {{ .Values.global.proxy.hash }}
  ssl: {{ .Values.global.proxy.ssl }}
{{- end }}
