{{- define "prometheus.server" -}}
{{- $logLevel := dict "root" $ "logLevels" (dict "trace" "debug" "debug" "debug" "info" "info" "warn" "warn" "error" "error" "crit" "error") "component" "prometheusServer" | include "stacklight.logLevel" -}}
server:
  global:
    scrape_interval: 15s
    scrape_timeout: 10s
  image:
    {{- include "stacklight.getImage" (dict "root" . "image" "prometheus") | nindent 4 }}
  affinity:
    podAntiAffinity:
      preferredDuringSchedulingIgnoredDuringExecution:
      - weight: 1
        podAffinityTerm:
          topologyKey: "kubernetes.io/hostname"
          labelSelector:
            matchExpressions:
            - key: app
              operator: In
              values:
              - "opensearch-master"
            - key: component
              operator: In
              values:
              - "master"
  {{- if .Values.openstack.enabled }}
      - weight: 1
        podAffinityTerm:
          topologyKey: "kubernetes.io/hostname"
          namespaces: ["{{ .Values.openstack.namespace }}"]
          labelSelector:
            matchExpressions:
            - key: application
              operator: In
              values:
              - "mariadb"
            - key: component
              operator: In
              values:
              - "server"
  {{- end }}
  {{- if .Values.highAvailabilityEnabled }}
      requiredDuringSchedulingIgnoredDuringExecution:
      - labelSelector:
          matchExpressions:
          - key: app
            operator: In
            values:
            - "prometheus"
          - key: component
            operator: In
            values:
            - "server"
        topologyKey: "kubernetes.io/hostname"
  {{- else }}
  strategy:
    type: RollingUpdate
    rollingUpdate:
      maxUnavailable: 1
      maxSurge: 0
  {{- end }}
  replicaCount: {{ get (include "stacklight.replicas" . | fromYaml) "prometheusServer" }}
  extraArgs:
    {{- if and .Values.cortexDev.enabled .Values.cortexDev.mimir }}
    enable-feature: agent
    {{- else }}
      {{- $prometheusRetentionTime := .Values.prometheusServer.retentionTime }}
      {{- $prometheusRetentionSize := .Values.prometheusServer.retentionSize }}
      {{- if .Values.cortexDev.enabled }}
        {{- $prometheusRetentionTime = "2h" }}
        {{- $prometheusRetentionSize = "2GB" }}
      {{- end }}
    storage.tsdb.path: /data
    storage.tsdb.retention.time: {{ $prometheusRetentionTime }}
    storage.tsdb.retention.size: {{ $prometheusRetentionSize }}
    rules.alert.resend-delay: {{ .Values.prometheusServer.alertResendDelay }}
    {{- end }}
  {{- if $logLevel }}
    "log.level": {{ $logLevel | quote }}
  {{- end }}
  persistentVolume:
    # keep persistency in agent mode to not lock the release update
    enabled: true
    size: {{ .Values.prometheusServer.persistentVolumeClaimSize }}
    mountPath: /data
  {{- if or .Values.storage.defaultStorageClass .Values.storage.componentStorageClasses.prometheusServer }}
    {{- $storageClass := .Values.storage.defaultStorageClass }}
    {{- if .Values.storage.componentStorageClasses.prometheusServer }}
      {{- $storageClass = .Values.storage.componentStorageClasses.prometheusServer }}
    {{- end }}
    storageClass: "{{ $storageClass }}"
  {{- end }}
{{- if .Values.cortexDev.enabled }}
  {{- include "stacklight.resources" (dict "inTplValues" .Values "service" "cortexPrometheus") | indent 2 }}
{{- else }}
  {{- include "stacklight.resources" (dict "inTplValues" .Values "service" "prometheusServer") | indent 2 }}
{{- end }}
  {{- include "stacklight.nodeSelector" (dict "inTplValues" .Values "service" "prometheusServer") | nindent 2 }}
  {{- include "stacklight.tolerations" (dict "inTplValues" .Values "service" "prometheusServer") | nindent 2 }}
  {{- if .Values.mke.enabled }}
  extraHostPathMounts:
  - name: mkecerts
    hostPath: "{{ .Values.mke.dockerdDataRoot }}/volumes/ucp-node-certs/_data"
    mountPath: /tmp/mkecerts
    subPath: ""
    readOnly: true
  {{- end }}
  {{- if .Values.prometheusServer.remoteWriteSecretMounts }}
  extraSecretMounts:
    {{- range $secretMount := .Values.prometheusServer.remoteWriteSecretMounts }}
  - name: "{{ $secretMount.secretName }}"
    mountPath: {{ $secretMount.mountPath }}
    readOnly: true
    secretName: "{{ $secretMount.secretName }}"
    subPath: ""
    reloadConfigMap: true
    {{- end }}
  {{- end }}
  {{- if .Values.exposeLoadBalancers }}
  service:
    type: LoadBalancer
  {{- end }}
{{- if not (empty .Values.clusterId) }}
clusterId: {{ .Values.clusterId }}
{{- end }}
serverFiles:
  prometheus.yml:
    scrape_configs:
      {{- include "prometheus.scrapeConfigs" . | nindent 6 }}
    {{- if or .Values.prometheusServer.remoteWrites .Values.cortexDev.enabled }}
    remote_write:
      {{- if .Values.prometheusServer.remoteWrites }}
    {{- .Values.prometheusServer.remoteWrites | toYaml | nindent 4 }}
      {{- end }}
      {{- if .Values.cortexDev.enabled }}
        {{- if .Values.cortexDev.mimir }}
    - url: "http://mimir-distributor:8080/api/v1/push"
        {{- else }}
    - url: "http://cortex-distributor:8080/api/v1/push"
        {{- end }}
      queue_config:
        capacity: 500
        retry_on_http_429: true
        max_samples_per_send: 100
      metadata_config:
        max_samples_per_send: 100
      {{- end }}
    {{- end }}
  {{- if not .Values.cortexDev.enabled }}
    rule_files:
    - /etc/config/rules
    - /etc/config/alerts
  rules:
    groups:
    {{- range $group, $groupItem := include "stacklight.recordingRules" . | fromYaml -}}
      {{ $groupItem | toYaml | nindent 4 -}}
    {{- end }}
  alerts:
    groups:
    - name: all
      rules:
    {{- range $alertName, $alertValues := include "stacklight.getEnabledAlerts" . | fromYaml }}
      - alert: {{ $alertName -}}
        {{ $alertValues | toYaml | nindent 8 -}}
    {{- end }}
  {{- end }}
{{- end }}

{{- define "prometheus.metricFilter" -}}
{{- if index .Values.metricFilter "enabled" }}
{{- $metricFilterNamespaces := list "kaas" "kube-system" "stacklight" }}
{{- if kindIs "map" .Values.metricFilter.namespaces }}
  {{- $metricFilterNamespaces = keys .Values.metricFilter.namespaces }}
{{- else if kindIs "slice" .Values.metricFilter.namespaces }}
  {{- $metricFilterNamespaces = .Values.metricFilter.namespaces }}
{{- end }}
metricFilter:
  cAdvisor:
  - action: {{ .Values.metricFilter.action }}
    labelRegexes:
      namespace: {{ $metricFilterNamespaces | sortAlpha | uniq | join "|" | quote }}
  kubeStateMetrics:
  - action: {{ .Values.metricFilter.action }}
    labelRegexes:
      namespace: {{ $metricFilterNamespaces | sortAlpha | uniq | join "|" | quote }}
{{- end -}}
{{- end -}}

{{- define "prometheus.relay" -}}
relay:
  enabled: {{ not .Values.cortexDev.enabled }}
  replicaCount: {{ get (include "stacklight.replicas" . | fromYaml) "prometheusRelay" }}
  env:
  - name: PrometheusRelayClientTimeout
    value: {{ .Values.prometheusRelay.clientTimeout | int64 | quote }}
  - name: PrometheusResponseLimitBytes
    value: {{ .Values.prometheusRelay.responseLimitBytes | int64 | quote }}
  image:
    {{- include "stacklight.getImage" (dict "root" . "image" "prometheusRelay") | nindent 4 }}
  {{- include "stacklight.resources" (dict "inTplValues" .Values "service" "prometheusRelay" ) | nindent 2 }}
  {{- include "stacklight.nodeSelector" (dict "inTplValues" .Values "service" "prometheusRelay") | nindent 2 }}
  {{- include "stacklight.tolerations" (dict "inTplValues" .Values "service" "prometheusRelay") | nindent 2 }}
{{- end -}}

{{- define "prometheus.alertsOverride" -}}
{{- if not (empty .Values.clusterId) }}
alertsCommonLabels:
  cluster_id: {{ .Values.clusterId }}
{{- end }}
{{- end -}}

{{- define "prometheus.initChownData" -}}
initChownData:
  image:
    {{- include "stacklight.getImage" (dict "root" . "image" "alpineUtils") | nindent 4 }}
{{- end }}

{{- define "prometheus.configmapReload" -}}
configmapReload:
  image:
    {{- include "stacklight.getImage" (dict "root" . "image" "configmapReload") | nindent 4 }}
{{- end }}

{{- define "prometheus.kubeStateMetrics" -}}
{{/* https://github.com/kubernetes/community/blob/master/contributors/devel/sig-instrumentation/logging.md */}}
{{- $logLevel := dict "root" $ "logLevels" (dict "trace" "5" "debug" "4" "info" "3" "warn" "1" "error" "0" "crit" "0") "component" "kubeStateMetrics" | include "stacklight.logLevel" -}}
kubeStateMetrics:
  args:
    "resources": "cronjobs,daemonsets,deployments,endpoints,jobs,namespaces,nodes,persistentvolumes,persistentvolumeclaims,pods,services,statefulsets"
  {{- if $logLevel }}
    "v": {{ $logLevel | quote }}
  {{- end }}
  replicaCount: {{ get (include "stacklight.replicas" . | fromYaml) "kubeStateMetrics" }}
  image:
    {{- include "stacklight.getImage" (dict "root" . "image" "kubeStateMetrics") | nindent 4 }}
  {{- include "stacklight.resources" (dict "inTplValues" .Values "service" "kubeStateMetrics") | nindent 2 }}
  {{- include "stacklight.nodeSelector" (dict "inTplValues" .Values "service" "kubeStateMetrics") | nindent 2 }}
  {{- include "stacklight.tolerations" (dict "inTplValues" .Values "service" "kubeStateMetrics") | nindent 2 }}
{{- end }}

{{- define "prometheus.iam" -}}
  {{- if .Values.iam.enabled }}
iam:
  enabled: true
    {{- if .Values.iam.tls.enabled }}
  proxyScheme: https
    {{- else }}
  proxyScheme: http
    {{- end }}
  {{- end }}
{{- end }}

{{- define "prometheus.init" -}}
initExternalIP:
  image:
    {{- include "stacklight.getImage" (dict "root" . "image" "alpineUtils") | nindent 4 }}
initYamlMerge:
  image:
    {{- include "stacklight.getImage" (dict "root" . "image" "yq") | nindent 4 }}
{{- end }}

{{- define "prometheus.pushgateway" -}}
pushgateway:
  enabled: false
{{- end -}}
